package thegame.server;

import java.io.*;
import java.net.*;
import java.util.*;
import thegame.shared.*;


/**
 * The connection to GameClient on the GameServer side.
 * @author Erki Suurjaak
 * @version 0.99 (2000/08/28)
 */
public class GameServerConnection
  extends GameConnection
{
  /** The GameServer the GameServerConnection sends messages to. */
  protected GameServer
  gameServer;

  /** The GameServerConnection referencing the remote player. */
  protected GameServerConnection
  remotePlayer;

  /** The name of the player on the GameClient side. */
  protected String
  playerName = "";

  /** Is true when the player is waiting for a joining player; is false otherwise. */
  protected boolean
  isWaiting = false;

  /** Is true if the player is drawing circles; is false if the player is drawing crosses. */
  protected boolean
  isPlayerDrawingCircles = false;

  /** Is true if the player is ready to receive a playerlist; is false otherwise. */
  protected boolean
  isReadyForPlayerList = false;


  /**
   * Constructs a GameServerConnection on the specified socket and binds it to the specified GameServer.
   * @param socket the socket to be used (must be already initialized)
   * @param gameServer  the GameServer to send messages to
   * @throws IOException if an I/O error occurs when constructing the superclass
   */
  public
  GameServerConnection(Socket socket, GameServer gameServer)
  throws IOException
    {
      super(socket);
      this.gameServer = gameServer;
      start();
    }


  /**
   * The socket is listened on and if a message arrives, it is sent either to gameServer, to remotePlayer, or is discarded.
   */
  public void
  run()
    {
      try
        {
        while (isRunning)
          {
            sleep(1000);
            String str = in.readLine();
            if (getMessageID(str) == SHUTDOWN_MESSAGE)
              {
                gameServer.handleShutdownMessage(this, getMessageBody(str));
                continue;
              }
            for (int i = 0 ; i < PLAYER_TO_SYSTEM_MESSAGES.length; i++)
              if (getMessageID(str) == PLAYER_TO_SYSTEM_MESSAGES[i])
                {
                  gameServer.handlePlayerMessage(this, str);
                  continue;
                }
            if (remotePlayer != null)
              remotePlayer.send(str);
          }
        }
      catch (Exception e)
        {
        } 
  }


  /**
   * Returns the isWaiting property.
   * @return the isWaiting property
   */
  public boolean
  isWaiting()
    {
      return isWaiting;
    }


  /**
   * Sets the isWaiting property.
   * @param isWaiting true if the player is set waiting; false otherwise
   */
  public void
  setWaiting(final boolean isWaiting)
    {
      this.isWaiting = isWaiting;
    }


  /**
   * Returns the remotePlayer property.
   * @return the remotePlayer property
   */
  public GameServerConnection
  getRemotePlayer()
    {
      return remotePlayer;
    }


  /**
   * Sets the remotePlayer property.
   * @param remotePlayer the GameServerConnection referencing the remote player
   */
  public void
  setRemotePlayer(GameServerConnection remotePlayer)
    {
      this.remotePlayer = remotePlayer;
    }


   /**
    * Sets the isPlayerDrawingCircles property.
    * @param isPlayerDrawingCircles true if the player is drawing circles; false it the player is drawing crosses
    */
  public void
  setPlayerDrawingCircles(final boolean isPlayerDrawingCircles)
    {
      this.isPlayerDrawingCircles = isPlayerDrawingCircles;
    }


  /**
   * Returns the isPlayerDrawingCircles property.
   * @return the isPlayerDrawingCircles property
   */
  public boolean
  getPlayerDrawingCircles()
    {
      return isPlayerDrawingCircles;
    }


  /**
   * Returns the isReadyForPlayerList property.
   * @return the isReadyForPlayerList property
   */
  public boolean
  isReadyForPlayerList()
    {
      return isReadyForPlayerList;
    }


  /**
   * Sets the isReadyForPlayerList property.
   * @param isReadyForPlayerList true if the player is ready for playerlist; false otherwise
   */
  public void
  setReadyForPlayerList(final boolean isReadyForPlayerList)
    {
      this.isReadyForPlayerList = isReadyForPlayerList;
    }


  /**
   * Sets the name of the player on the GameClient side
   * @param playerName the name of the player on the GameClient side
   */
  public void
  setPlayerName(final String playerName)
    {
      this.playerName = playerName;
    }


  /**
   * Returns the name of the player on the GameClient side
   * @return the name of the player on the GameClient side
   */
  public String
  getPlayerName()
    {
      return playerName;
    }


}
