package thegame.server;

import java.io.*;
import java.net.*;
import java.util.*;
import thegame.shared.*;


/**
 * A utility to shut down the game server.
 * @author Erki Suurjaak
 * @version 0.99 (2000/08/28)
 */
public class GameServerKiller
  extends GameConnection
{
  /** The message shown when run with an insufficient number of command-line arguments. */
  public static final String
  USAGE_MESSAGE = "usage: KillGameServer <server_address> [port]\n\r"
                  + "where: host_address - the IP address of the server. Compulsory.\n\r"
                  + "       port         - the TCP port of the server. Optional - if not specified,\n\r"
                  + "                      the default port is used.";

  /** The message shown when a communications error occurs. */
  public static final String
  COMMUNICATIONS_ERROR_MESSAGE = "There was an error while trying to communicate with the server.";

  /** The password prompt shown at startup. */
  public static final String
  STARTUP_PASSWORD_PROMPT_MESSAGE = "Enter administrator password to be used: ";


  /**
   * Waits for a server response. Shows the message sent by the server and exits.
   */
  public void
  run()
    {
      while (true)
        {
          try
            {
                sleep(100);
                String str = in.readLine();
                if (str.length() != 0)
                  {
                    System.out.println(str);
                    System.exit(0);
                  }
            }
          catch (Exception e)
            {
            } 
        }
    }


  /**
   * Constructs a GameServerKiller. Asks for an administrator password to use, sends a shutdown message to the server and waits for a response.
   * @param socket the socket to use
   */
  public
  GameServerKiller(Socket socket)
  throws IOException
    {
      super(socket);
      String administratorPassword = "";
      BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
      do
        {
          System.out.print(STARTUP_PASSWORD_PROMPT_MESSAGE);
          administratorPassword = stdin.readLine().trim();
        }
      while (administratorPassword.length() == 0);
      send(SHUTDOWN_MESSAGE, administratorPassword);
      run();
    }


  /**
   * Builds a GameServerKiller. The first argument must be the IP address of the server, the second argument may be the TCP port of the server (if unspecified, the default port will be used).
   * @args the command-line arguments
   */
  public static void
  main(String[] args)
    {
      if (args.length != 1)
        {
          System.out.println(USAGE_MESSAGE);
          System.exit(0);
        }
      String host = args[0];
      int port = DEFAULT_SERVER_PORT;
      if (args.length > 1)
        {
          try
            {
              port = Integer.valueOf(args[1]).intValue();
            }
          catch (NumberFormatException e)
            {
            }
        }
      try
        {
          new GameServerKiller(new Socket(host, port));
        }
      catch (IOException e)
        {
          System.err.println(COMMUNICATIONS_ERROR_MESSAGE);
          System.exit(1);
        }
    }


}
